% FIXED PARAMETERS, DO NOT MODIFY!
% Initialize DJI M600 Pro hexacopter model with
% physical and control parameters
% Selectable configuration regarding mass and inertias
% NED ground coordinate system and aligned (X forward, Y right, Z down)
% body system are applied
%
% Institute for Computer Science and Control,  Budapest, Hungary
% Systems and Control Laboratory 2023.
% v3.0 12. June 2023.
%

% simulation time step
ts_hexacopter = 0.005;

% Graviatational constant
params.g = 9.80665;

% Air density (ISA sea level)
params.rho = 1.225;

switch INERTflag
    case 1
        % TB47S batteries
        params.m = 9.530;
        
        % CAD model data (without equipment, TB47S battery)
        params.Jxx = 6.894e6/10^7;
        params.Jyy = 5.971e6/10^7;
        params.Jzz = 1.275;
        
    case 2
        % TB48S batteries
        params.m = 10.04;
        
        % CAD model data (without equipment, TB48S battery)
        params.Jxx = 6.949e6/10^7;
        params.Jyy = 6.026e6/10^7;
        params.Jzz = 1.286;    
end

% BLDC rotor + propeller inertia
% (source: E2000 Pro Tuned Propulsion System User Manual
% https://dl.djicdn.com/downloads/e2000/20161220/E2000+Pro+User+Manual+V1.4.pdf
% @2023)
params.Jr = 7e-4;

% Engine maximum speed parameter:
% According to test figure on page 9 of E2000 Pro Tuned Propulsion System User Manual
% https://dl.djicdn.com/downloads/e2000/20161220/E2000+Pro+User+Manual+V1.4.pdf
% @2023 and flight test results
params.engine_saturation=456.45;

% Hover engine angular speeds
params.Om0=283.032;

% TRANSLATIONAL model parameters:
params.A=0.2231;            % propeller area
params.R=0.2665;            % propeller radius
params.cT0=0.0105;          % Hover thrust coefficient
params.Khx=0.94;            % X hub force coefficient
params.Kx=0.072;            % X air drag coefficient
params.Khy=0.7641;          % Y hub force coefficient
params.Ky=0.0663;           % Y air drag coefficient

% ROTATIONAL model parameters:
params.l=0.60285;           % motor arm length
params.cP0=0.2795;          % power coefficient
params.Kr=0.3542;           % Z axis rotational air drag coefficient

%% CONTROLLER parameters
% Altitude (vertical speed) controller
% Integral control in transfer function form
% from vertical speed error [m/s] to engine angular rate [rad/s]
Alt_num=-[2 1]*18;
Alt_den=[0.1 1 0];
% Down position error [m] to vertical speed [m/s] reference gain
Alt_err=0.3817;

% Yaw (yawrate) controller in transfer function form
% from yawrate error [rad/s] to engine angular rate [rad/s]
Eg=200;    % Gain
Ez=0.3;     % numerator zero
ET=0.06;  % Time constant
Yaw_num=Eg*[1 Ez];
Yaw_den=[ET 1 0];
% Yaw angle error [rad] to yawrate reference [rad/s] gain
P_yaw=2.5;

% Position error [m] to speed reference [m/s] gains
T_err=0.5;  % Tangential
N_err=0.2;  % Normal

% Pitch (angle) controller in transfer function form
% from pitch angle error [rad] to engine angular rate [rad/s]
Pnum=100*[0.25 5];
Pden=[0.01 1];

% Roll (angle) controller in transfer function form
% from roll angle error [rad] to engine angular rate [rad/s]
Rnum=100*[0.32 5];
Rden=[0.01 1];

% Vertical speed limits [m/s] (M600 specs)
UvD=5;
DvD=-5;

% Horizontal speed limits [m/s] (M600 specs 18 m/s w/o wind)
% Tangential
UvT=17;
DvT=-17;
% Normal
UvN=17;
DvN=-17;

% Pitch and roll angle limits [rad] (M600 specs):
pitch_max=25/180*pi;
roll_max=25/180*pi;

% yawrate limits [rad/s] (M600 specs 150 deg/s)
r_max=2.618;
r_min=-2.618;

% Pitch reference [rad] from velocity error [m/s] gains:
PP=-0.07;
PI=-0.03;

% Roll reference [rad] from velocity error [m/s] gains:
RP=0.07;
RI=0.03;